#ifndef DEVICE_H
#define DEVICE_H

#include <flydevices_export.h>

#include <QSharedPointer>
#include <QHash>
#include <QString>
#include <QVariant>

class QUdevDevice;

class IDeviceDescriptor;
class IDeviceBackend;

class FLY_DEVICES_EXPORT Device
{
public:
    Device();
    Device(QSharedPointer<QUdevDevice> device);
    Device(QSharedPointer<IDeviceBackend> backend);

    bool operator==(const Device &other) const;
    bool operator!=(const Device &other) const;

    bool isNull() const;
    QString udi() const;

    QString product() const;
    QString vendor() const;

    QString presentationName() const;
    QString icon() const;
    QStringList emblems() const;
    QString description() const;

    QVariant value(const QString &property, bool decode = true) const;
    QString subsystem() const;
    QString model() const;
    QString syspath() const;
    QString sysname() const;
    QString type() const;
    QString driver() const;
    QString sysnum() const;
    QString devnode() const;
    Device parentDevice() const;
    bool isValid() const;

    QString category() const;

    QString moduleName() const;

    using Properties = QHash<QString, QVariant>;
    Properties propertiesAll() const;
    Properties propertiesUdev() const;
    Properties propertiesBackend() const;

    void setBackend(QSharedPointer<IDeviceBackend> backend);

private:
    QSharedPointer<QUdevDevice> m_device;
    QSharedPointer<IDeviceBackend> m_backend;
    IDeviceDescriptor *m_deviceDescriptor;
    mutable QString m_moduleName;
};

#endif // DEVICE_H
