#ifndef IDEVICEBACKEND_H
#define IDEVICEBACKEND_H

#include <flydevices_export.h>

#include <QStringList>
#include <QPair>
#include <QSharedPointer>

class FLY_DEVICES_EXPORT IDeviceBackend
{
public:
    virtual ~IDeviceBackend() = default;
    IDeviceBackend() = default;
    IDeviceBackend(const IDeviceBackend &) = delete;
    IDeviceBackend &operator=(const IDeviceBackend &) = delete;

    virtual QSharedPointer<IDeviceBackend> createBackend(const QString &udi) const = 0;

    virtual QString udi() const = 0;
    virtual QString nativePath() const = 0;
    virtual QString parentUdi() const = 0;

    virtual QString product() const = 0;
    virtual QString vendor() const = 0;

    virtual QString presentationName() const = 0;
    virtual QString icon() const = 0;
    virtual QStringList emblems() const
    {
        return QStringList();
    }

    using DescriptionLine = QPair<QString, QString>;
    using DescriptionLines = QList<DescriptionLine>;
    virtual DescriptionLines descriptionLines() const
    {
        return DescriptionLines();
    }

    virtual QString category() const
    {
        return QString();
    }

    virtual bool hasProperty(const QString &name) const = 0;
    virtual QVariant property(const QString &name) const = 0;
    virtual QHash<QString, QVariant> properties() const = 0;
};

#endif // IDEVICEBACKEND_H
